///File : LetterGameBase.cpp
//------------------------------------------------------------------------------
/*
    This file is part of LetterGames.

    LetterGames is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    LetterGames is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with LetterGames.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "LetterGameBase.hpp"
//------------------------------------------------------------------------------

LetterGameBase::LetterGameBase()
{
    m_isOk = false;
    srand(time(NULL)); // NE JAMAIS L'OUBLIER ==> Pour hasard()
    m_LongestWordLen = 0;
}
//------------------------------------------------------------------------------
int LetterGameBase::Hasard(int min, int max) const
{
    return (int) (min + ((float) rand() / RAND_MAX * (max - min + 1)));
}
//------------------------------------------------------------------------------
bool LetterGameBase::IsOk() const
{
    return m_isOk;
}

//------------------------------------------------------------------------------
void LetterGameBase::Init(const wxString &defaultLanguage)
{
    wxTextFile file;
    if(file.Open(findFile(_T("data/available_words_list"))))
    {
        m_availableLanguagesList.clear();

        wxString str = file.GetFirstLine();
        while(!file.Eof())
        {
            StructAvailableLanguages languageStr;
            languageStr.realName = str;

            str = file.GetNextLine();
            languageStr.name = str;
            m_availableLanguagesList.push_back(languageStr);

            str = file.GetNextLine();
        }
    }
    else
    {
        wxMessageBox(_("Error : Can't get words list !\nWill now exit."), _("Error..."), wxOK | wxICON_ERROR);
        exit(1);
    }

    // Selection langue
    if(IsLanguageAvailable(defaultLanguage))
    {
        m_defaultLanguage = defaultLanguage;
    }
    else
    {
        m_defaultLanguage = m_availableLanguagesList[0].name;
    }

    LoadWords(m_defaultLanguage);

    m_isOk = true;
}
//------------------------------------------------------------------------------
bool LetterGameBase::IsLanguageAvailable(const wxString &language) const
{
    for(int i = 0; i < m_availableLanguagesList.size(); i++)
    {
        if(language == m_availableLanguagesList[i].name)
        {
            return true;
        }
    }

    return false;
}
//------------------------------------------------------------------------------
wxString LetterGameBase::GetLanguageRealName(const wxString &language) const
{
    for(int i = 0; i < m_availableLanguagesList.size(); i++)
    {
        if(language == m_availableLanguagesList[i].name)
        {
            return m_availableLanguagesList[i].realName;
        }
    }

    return _T("");
}
//------------------------------------------------------------------------------
wxString LetterGameBase::GetCurrentLanguage() const
{
    return m_currentLanguage;
}
//------------------------------------------------------------------------------
int LetterGameBase::GetWordListCount() const
{
    return m_wordsTbl.size();
}
//-----------------------------------------------------------------------------
wxString LetterGameBase::GetEveryUserLetter() const
{
    wxString list;
    for(int letter = 'A'; letter <= 'Z'; letter++)
    {
        list << GetLettersFor(letter);
    }

    return list;
}
//-----------------------------------------------------------------------------
wxString LetterGameBase::GetLettersFor(const wxChar &letter) const
{
    wxString letters;

    letters << letter;

    switch (letter)
    {
        case 'A':
        case 'a':
            letters << _T("àäâ");
            break;
        case 'C':
        case 'c':
            letters << _T("ç");
            break;
        case 'E':
        case 'e':
            letters << _T("éèêë");
            break;
        case 'I':
        case 'i':
            letters << _T("îï");
            break;
        case 'O':
        case 'o':
            letters << _T("ôö");
            break;
        case 'U':
        case 'u':
            letters << _T("üûù");
            break;
    }

    letters = letters.Lower() + letters.Upper();

    return letters;
}
//------------------------------------------------------------------------------
wxString LetterGameBase::GetLanguageName(const wxString &language) const
{
    for(int i = 0; i < m_availableLanguagesList.size(); i++)
    {
        if(language == m_availableLanguagesList[i].realName)
        {
            return m_availableLanguagesList[i].name;
        }
    }

    return _T("");
}
//------------------------------------------------------------------------------
wxArrayString LetterGameBase::GetLanguageRealNameList() const
{
    wxArrayString list;

    list.Alloc(m_availableLanguagesList.size()); // Pr perf
    for(int i = 0; i < m_availableLanguagesList.size(); i++)
    {
        list.Add(m_availableLanguagesList[i].realName);
    }

    return list;
}
//------------------------------------------------------------------------------
void LetterGameBase::LoadWords(const wxString &language)
{
    wxString filename = _T("data/words_list_") + language;
    wxTextFile file;
    m_LongestWordLen = 0;

    if(file.Open(findFile(findFile(filename))))
    {
        m_wordsTbl.clear();
        m_currentLanguage = language;

        for ( wxString str = file.GetFirstLine(); !file.Eof(); str = file.GetNextLine() )
        {
            m_wordsTbl.push_back(str);

            // On prend la taille du plus long mot...
            if(str.Len() > m_LongestWordLen)
            {
                m_LongestWordLen = str.Len();
            }
        }
    }
    else
    {
        wxMessageBox(_("Error : Can't get words list !\nWill now exit."), _("Error..."), wxOK | wxICON_ERROR);
        exit(1);
    }
}
