///File : MainPanel.cpp
//------------------------------------------------------------------------------
/*
    This file is part of HangMan.

    HangMan is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    HangMan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with HangMan.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "MainPanel.hpp"
//------------------------------------------------------------------------------
MainPanel::MainPanel( wxWindow* parent, wxWindowID id, HangMan *hangMan, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
{
    // HangMan
    m_HangMan = hangMan;

    m_currentImgId = -1;

    // Controls
	wxBoxSizer *MainSizer = new wxBoxSizer( wxHORIZONTAL );

    wxBoxSizer *LettersWordSizer = new wxBoxSizer( wxVERTICAL );

    // 0 : Comments
    wxBoxSizer *CommentSizer = new wxBoxSizer( wxHORIZONTAL );

    CommentSizer->AddStretchSpacer(1); // Pour centrer sous wxGTK (bug)

    m_CommentsLabel = new wxStaticText(this, wxID_ANY, _T(""), wxDefaultPosition, wxDefaultSize, wxALIGN_CENTER);
    m_CommentsLabel->SetFont(wxFont(16, wxFONTFAMILY_DECORATIVE, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD));
    m_CommentsLabel->SetForegroundColour(*wxBLUE);
    CommentSizer->Add(m_CommentsLabel, 0, 0, 0);

    CommentSizer->AddStretchSpacer(1);


    LettersWordSizer->Add(CommentSizer, 0, wxEXPAND | wxALL, 5);

    // 1 : Mot
    LettersWordSizer->AddStretchSpacer(1);

    wxBoxSizer *WordSizer = new wxBoxSizer( wxHORIZONTAL );

    WordSizer->AddStretchSpacer(1); // Pour centrer sous wxGTK (bug)

    m_CurrentWord = new wxStaticText(this, wxID_ANY, _T(""));
    m_CurrentWord->SetFont(wxFont(48, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL));
    WordSizer->Add(m_CurrentWord, 0, 0, 0);

    WordSizer->AddStretchSpacer(1);

    LettersWordSizer->Add(WordSizer, 0, wxEXPAND, 0);


    LettersWordSizer->AddStretchSpacer(1);

    // 2 : Letters
    wxGridSizer *LettersSizer = new wxGridSizer(5, 6, 5, 5);

    for(int i = 0; i < LETTERS_NB; i++)
    {
        wxString letterTxt;
        letterTxt << wxChar('A' + i);

        m_LettersTbl[i] = new wxButton(this, wxID_ANY, letterTxt, wxDefaultPosition, wxSize(-1, 40));

        LettersSizer->Add(m_LettersTbl[i], 0, wxEXPAND, 0);

        m_LettersTbl[i]->Connect( wxID_ANY, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainPanel::OnLetterBt), NULL, this);
        m_LettersTbl[i]->Connect( wxID_ANY, wxEVT_KEY_DOWN, wxKeyEventHandler(MainPanel::OnKeyPress), NULL, this);
    }
    LettersWordSizer->Add(LettersSizer, 0, wxEXPAND | wxALL, 5);

	MainSizer->Add(LettersWordSizer, 1, wxEXPAND, 0);

	// Line
    MainSizer->Add(new wxStaticLine(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_VERTICAL), 0, wxEXPAND | wxTOP | wxBOTTOM | wxRIGHT, 5 );

	// Partie droite : dessin + text + newBt
	wxSizer *ScoreSizer = new wxBoxSizer( wxVERTICAL );

    ScoreSizer->AddStretchSpacer(1);

    m_HangManImg = new wxStaticBitmap(this, wxID_ANY, wxBitmap());
    ScoreSizer->Add(m_HangManImg, 0, wxALL | wxALIGN_CENTER, 5);

    ScoreSizer->AddStretchSpacer(1);

    m_Score = new wxStaticText(this, wxID_ANY, _T(""));
    ScoreSizer->Add(m_Score, 0, wxALL, 5);

    #if defined(__WXMSW__) // Bug pour Windows : dès qu'on appuit sur n ==> Valider.
        m_NewWordBt = new wxButton(this, wxID_NEW, _("New"), wxDefaultPosition, wxSize(-1, 50));
    #else
        m_NewWordBt = new wxButton(this, wxID_NEW, _T(""), wxDefaultPosition, wxSize(-1, 50));
    #endif //#ifdef WIN32
    ScoreSizer->Add(m_NewWordBt, 0, wxALL | wxEXPAND, 5);

    MainSizer->Add(ScoreSizer, 0, wxEXPAND);

    // Finition
	SetSizer(MainSizer);

	// Preload des images...
	for(int i = 0; i <= HANGMAN_NB_CHANCE; i++)
	{
        wxString ImgFileName = findFile(wxString::Format(_T("data/hangman%d.png"), i));
        if(wxFileExists(ImgFileName))
        {
            m_HangManImgTbl[i].LoadFile(ImgFileName, wxBITMAP_TYPE_PNG);
        }

	}

	// Events
	m_NewWordBt->Connect( wxID_ANY, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainPanel::OnNewWordBt), NULL, this);
    m_NewWordBt->Connect( wxID_ANY, wxEVT_KEY_DOWN, wxKeyEventHandler(MainPanel::OnKeyPress), NULL, this);

    // Update info
    NewWord();
}
//------------------------------------------------------------------------------
void MainPanel::OnKeyPress(wxKeyEvent& event)
{
    if(!(m_HangMan->IsWon() || m_HangMan->IsLost()))
    {
        for(wxChar letter = 'A'; letter <= 'Z'; letter++)
        {
            if(event.GetKeyCode() == letter)
            {
                if(m_HangMan->GetTriedLetters().Find(letter) == wxNOT_FOUND) // Si lettre pas encore tésté...
                {
                    m_HangMan->TryLetter(letter);
                }

                MyRefresh();
                SetFocusToFirstLetter();

                break;
            }
        }
    }

    event.Skip();
}
//------------------------------------------------------------------------------
void MainPanel::OnLetterBt(wxCommandEvent& event)
{
    wxChar letter = '\0';

    for(int i = 0; i < LETTERS_NB; i++)
    {
        if(event.GetId() == m_LettersTbl[i]->GetId())
        {
            letter = m_LettersTbl[i]->GetLabel()[0];

            m_HangMan->TryLetter(letter);
            MyRefresh();

            if( !(m_HangMan->IsWon() || m_HangMan->IsLost()))
            {
                // Donne le focus au suivant...
                bool focusOk = false;
                for(int j = i; j < LETTERS_NB; j++) // On commence à la lettre en cours...
                {
                    if(m_LettersTbl[j]->IsEnabled())
                    {
                        m_LettersTbl[j]->SetFocus();
                        focusOk = true;
                        break;
                    }
                }

                if(!focusOk) // Si le focus n'est pas donné dans les derniers ==> Recomance au début
                {
                    SetFocusToFirstLetter();
                }
            }

            break;
        }
    }

}
//------------------------------------------------------------------------------
void MainPanel::OnNewWordBt(wxCommandEvent& event)
{
    NewWord();
}
//------------------------------------------------------------------------------
/*void MainPanel::ResizeWord()
{
    int i = 1, lastSize;

    while(1)
    {
        lastSize = i;
        i++;

        m_CurrentWord->SetFont(wxFont(i, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL));



        wxString str;
        str << GetSize().GetWidth();
        wxMessageBox(str);
        GetSizer()->Layout();

        if(m_CurrentWord->GetSize().GetWidth() > m_CurrentWord->GetContainingSizer()->GetMinSize().GetWidth())
        {
            m_CurrentWord->SetFont(wxFont(lastSize, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL));
            break;
        }

    }
}*/
//------------------------------------------------------------------------------
void MainPanel::NewWord()
{
    m_HangMan->NewWord();
    MyRefresh();
    SetFocusToFirstLetter();
}
//------------------------------------------------------------------------------
void MainPanel::SetFocusToFirstLetter()
{
    // Focus pour le premier enable...
    for(int i = 0; i < LETTERS_NB; i++)
    {
        if(m_LettersTbl[i]->IsEnabled())
        {
            m_LettersTbl[i]->SetFocus();
            break;
        }
    }
}
//------------------------------------------------------------------------------
void MainPanel::MyRefresh()
{
    if(m_HangMan->IsWon() || m_HangMan->IsLost())
    {
        // Désactivation de tous les bouttons
        for(int i = 0; i < LETTERS_NB; i++)
        {
            m_LettersTbl[i]->Disable();
        }

        m_CurrentWord->SetLabel(m_HangMan->GetCurrentWordResult());

        if(m_HangMan->IsWon())
        {
            m_CurrentWord->SetForegroundColour(*wxGREEN);
        }
        else
        {
            m_CurrentWord->SetForegroundColour(*wxRED);
        }

        // Comments
        wxString comment;
        switch(m_HangMan->GetChanceUsed())
        {
            case 0:
                comment = _("Well done !");
                break;
            case 1:
            case 2:
            case 3:
                comment = _("Congratulation !");
                break;
            case 4:
            case 5:
                comment = _("Good.");
                break;
            case 6:
                comment = _("just in time !");
                break;
            case HANGMAN_NB_CHANCE:
                comment = _("Too bad !");
                break;
            default:
                comment = _T("");
                break;
        }

        if(comment != _T(""))
        {
            m_CommentsLabel->SetLabel(comment);
            m_CommentsLabel->Show();
        }

        // Focus pour le Bt nouveau
        m_NewWordBt->SetFocus();
    }
    else
    {
        m_CommentsLabel->Hide();
        m_CurrentWord->SetForegroundColour(*wxBLACK);

        // Désactivation des bouttons inutils
        for(int i = 0; i < LETTERS_NB; i++)
        {
            if(m_HangMan->GetTriedLetters().Find(m_LettersTbl[i]->GetLabel()[0]) != wxNOT_FOUND)
            {
                m_LettersTbl[i]->Disable();
            }
            else
            {
                m_LettersTbl[i]->Enable();
            }
        }

        m_CurrentWord->SetLabel(m_HangMan->GetCurrentWord());
    }

    // Score
    m_Score->SetLabel(wxString::Format(_("Chance : %d/%d\nScore : %d succeeded words on %d."), m_HangMan->GetChanceUsed(), HANGMAN_NB_CHANCE, m_HangMan->GetNbSucceededWords(), m_HangMan->GetNbPlayedWords()));

    // Img
    if(m_currentImgId != m_HangMan->GetChanceUsed()) // Si il faut recharger l'img
    {
        m_HangManImg->SetBitmap(wxBitmap(m_HangManImgTbl[m_HangMan->GetChanceUsed()]));
        m_HangManImg->SetSize(wxSize(m_HangManImgTbl[m_HangMan->GetChanceUsed()].GetWidth(), m_HangManImgTbl[m_HangMan->GetChanceUsed()].GetHeight()));

        m_currentImgId = m_HangMan->GetChanceUsed();
    }

    Layout();
}
